Package: BAMBI
Type: Package
Title: Bivariate Angular Mixture Models
Version: 2.3.5
Date: 2023-03-08
Author: Saptarshi Chakraborty,
        Samuel W.K. Wong
Maintainer: Saptarshi Chakraborty <chakra.saptarshi@gmail.com>
Description: Fit (using Bayesian methods) and simulate mixtures of univariate and bivariate angular distributions. Chakraborty and Wong (2021) <doi:10.18637/jss.v099.i11>.
License: GPL-3
LazyData: TRUE
RoxygenNote: 7.2.3
LinkingTo: Rcpp, RcppArmadillo
Imports: stats, stats4, graphics, lattice, grDevices, Rcpp, tcltk,
        qrng, mvtnorm, gtools, parallel, label.switching, methods,
        coda, future.apply, loo (>= 2.4.1), RColorBrewer,
        bridgesampling, scales, numDeriv
Suggests: future, gridExtra
Depends: R (>= 3.2.0)
URL: https://doi.org/10.18637/jss.v099.i11
BugReports: https://github.com/c7rishi/BAMBI/issues
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2023-03-08 22:28:25 UTC; chakrab2
Repository: CRAN
Date/Publication: 2023-03-08 23:10:05 UTC
Built: R 4.2.0; aarch64-apple-darwin20; 2023-07-11 00:42:26 UTC; unix
Archs: BAMBI.so.dSYM
