Package: BAMBI
Type: Package
Title: Bivariate Angular Mixture Models
Version: 2.3.7
Date: 2026-05-05
Authors@R: c(person(given = "Saptarshi",
                      family = "Chakraborty",
                      role = c("aut", "cre"),
                      email = "chakra.saptarshi@gmail.com"),
               person(given = c("Samuel", "W.K."),
                      family = "Wong",
                      role = "aut"),
               person(given = "Florence",
                      family = "Bockting",
                      role = "ctb", 
                      comment = "Patch for compatibility with {loo}"))
Maintainer: Saptarshi Chakraborty <chakra.saptarshi@gmail.com>
Description: Fit (using Bayesian methods) and simulate mixtures of univariate and bivariate angular distributions. Chakraborty and Wong (2021) <doi:10.18637/jss.v099.i11>.
License: GPL-3
LazyData: TRUE
RoxygenNote: 7.3.3
LinkingTo: Rcpp, RcppArmadillo
Imports: stats, stats4, graphics, lattice, grDevices, Rcpp, tcltk,
        qrng, mvtnorm, gtools, parallel, label.switching, methods,
        coda, future.apply, loo (>= 2.4.1), RColorBrewer,
        bridgesampling, scales, numDeriv
Suggests: future, gridExtra
Depends: R (>= 3.2.0)
URL: https://doi.org/10.18637/jss.v099.i11
BugReports: https://github.com/c7rishi/BAMBI/issues
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-05-08 20:25:41 UTC; chakrab2
Author: Saptarshi Chakraborty [aut, cre],
  Samuel W.K. Wong [aut],
  Florence Bockting [ctb] (Patch for compatibility with {loo})
Repository: CRAN
Date/Publication: 2026-05-09 06:02:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-09 06:27:57 UTC; unix
Archs: BAMBI.so.dSYM
