# Generated by roxygen2: do not edit by hand

S3method(as.mcmc.list,angmcmc)
S3method(bridge_sampler,angmcmc)
S3method(contour,angmcmc)
S3method(densityplot,angmcmc)
S3method(logLik,angmcmc)
S3method(loo,angmcmc)
S3method(plot,angmcmc)
S3method(print,angmcmc)
S3method(print,stepfit)
S3method(print,summary_angmcmc)
S3method(quantile,angmcmc)
S3method(summary,angmcmc)
S3method(waic,angmcmc)
export(DIC)
export(add_burnin_thin)
export(angmcmc)
export(bestcriterion)
export(bestmodel)
export(circ_cor)
export(circ_varcor_model)
export(contour_model)
export(d_fitted)
export(dvm)
export(dvmcos)
export(dvmcosmix)
export(dvmmix)
export(dvmsin)
export(dvmsinmix)
export(dwnorm)
export(dwnorm2)
export(dwnorm2mix)
export(dwnormmix)
export(extractsamples)
export(fit_angmix)
export(fit_incremental_angmix)
export(fit_vmcosmix)
export(fit_vmmix)
export(fit_vmsinmix)
export(fit_wnorm2mix)
export(fit_wnormmix)
export(fix_label)
export(is.angmcmc)
export(latent_allocation)
export(lpdtrace)
export(minuspi_to_pi)
export(paramtrace)
export(pointest)
export(r_fitted)
export(rvm)
export(rvmcos)
export(rvmcosmix)
export(rvmmix)
export(rvmsin)
export(rvmsinmix)
export(rwnorm)
export(rwnorm2)
export(rwnorm2mix)
export(rwnormmix)
export(select_chains)
export(surface_model)
export(vm2_mle)
export(zero_to_2pi)
import(graphics)
import(stats)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(bridgesampling,bridge_sampler)
importFrom(coda,as.mcmc.list)
importFrom(future.apply,future_lapply)
importFrom(grDevices,colorRampPalette)
importFrom(gtools,rdirichlet)
importFrom(lattice,densityplot)
importFrom(lattice,wireframe)
importFrom(loo,loo)
importFrom(loo,waic)
importFrom(methods,is)
importFrom(mvtnorm,rmvnorm)
importFrom(parallel,detectCores)
importFrom(qrng,sobol)
importFrom(stats4,mle)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
useDynLib(BAMBI, .registration = TRUE)
