# Bambi Community Code of Conduct

Be kind to others. Do not insult or put down others.
Behave professionally. Remember that harassment and sexist, racist,
or exclusionary jokes are not appropriate.

All communication should be appropriate for a professional audience
including people of many different backgrounds. Sexual language and
imagery are not appropriate.

Bambi is dedicated to providing a harassment-free community for everyone,
regardless of gender, sexual orientation, gender identity, and
expression, disability, physical appearance, body size, race,
or religion. We do not tolerate harassment of community members
in any form.

Thank you for helping make this a welcoming, friendly community for all.
